// -*- C++ -*-

/* 
 * GChemPaint
 * gchempaint-bonobo.cc 
 *
 * Copyright (C) 2001-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include "gchempaint-bonobo.h"
#include "file.h"
#include "filesel.h"
#include "tools.h"
#include <libgnome/libgnome.h>

/*******************************************************************************
 * Persist Stream implementation
 ******************************************************************************/
#define PERSIST_STREAM_TYPE         (persist_stream_get_type ())
#define PERSIST_STREAM(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), PERSIST_STREAM_TYPE, PersistStream))
#define PERSIST_STREAM_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), PERSIST_STREAM_TYPE, PersistStreamClass))
#define PERSIST_STREAM_IS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), PERSIST_STREAM_TYPE))
#define PERSIST_STREAM_IS_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), PERSIST_STREAM_TYPE))
#define PERSIST_STREAM_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), PERSIST_STREAM_TYPE, PersistStreamClass))

typedef struct {
	BonoboPersist parent;
	gcpDocument* pDoc;
} PersistStream;

typedef struct {
	BonoboPersistClass parent_class;

	POA_Bonobo_PersistStream__epv epv;
} PersistStreamClass;

GType persist_stream_get_type (void);

static void on_load(gpointer p, Bonobo_Stream stream, const Bonobo_Persist_ContentType type, CORBA_Environment  *ev)
{
	PersistStream *ps = (PersistStream*)bonobo_object(p);
	if (!ps->pDoc) return;
	CORBA_long l = Bonobo_Stream_seek(stream, 0, Bonobo_Stream_SeekEnd, ev);
//	CORBA_long l = bonobo_stream_client_get_length (stream, ev);//Seeme it does not always work!
	Bonobo_Stream_seek(stream, 0, Bonobo_Stream_SeekSet, ev);
	Bonobo_Stream_iobuf*  buffer;
	Bonobo_Stream_read(stream, l, &buffer, ev);
	char buf[l+1];
	memcpy(buf, buffer->_buffer, l);
	buf[l] = 0;
	xmlDocPtr xml = xmlParseMemory(buf, l);
	CORBA_free (buffer);
	if (xml) ps->pDoc->ParseXMLTree(xml);
}

static void on_save(gpointer p, Bonobo_Stream stream, const Bonobo_Persist_ContentType type, CORBA_Environment  *ev)
{
	PersistStream *ps = (PersistStream*)bonobo_object(p);
	if (!ps->pDoc) return;
	xmlDocPtr xml = ps->pDoc->BuildXMLTree();
	if (xml)
	{
		char* tmp;
		int size;
		xmlDocDumpMemory(xml, (xmlChar**)(&tmp), &size);
		if (tmp)
		{
			bonobo_stream_client_write (stream, tmp, size, ev);
			xmlFree((xmlChar*)(tmp));
		}
		xmlFreeDoc(xml);
	}
}

static PersistStream* persist_stream_new()
{
	PersistStream* ps = (PersistStream*) g_object_new (PERSIST_STREAM_TYPE, NULL);
	return ps;
}

static void persist_stream_init(PersistStream *ps)
{
}

static void persist_stream_class_init(PersistStreamClass *klass)
{
	POA_Bonobo_PersistStream__epv* epv = &(klass->epv);
	epv->load = (void(*)(void*, CORBA_Object_type*, const CORBA_char*, CORBA_Environment*)) on_load;
	epv->save = (void(*)(void*, CORBA_Object_type*, const CORBA_char*, CORBA_Environment*)) on_save;
}

 BONOBO_TYPE_FUNC_FULL (
	PersistStream,                /* Glib class name */
	Bonobo_PersistStream,  /* CORBA interface name */
	BONOBO_TYPE_PERSIST,  /* parent type */
	persist_stream);               /* local prefix ie. 'echo'_class_init */

#define GCP_BONOBO_CONTROL_TYPE           (gcp_bonobo_control_get_type ())
#define GCP_BONOBO_CONTROL(o)             (G_TYPE_CHECK_INSTANCE_CAST ((o), GCP_BONOBO_CONTROL_TYPE, gcpBonoboControl))
#define GCP_BONOBO_CONTROL_CLASS(k)       (G_TYPE_CHECK_CLASS_CAST((k), GCP_BONOBO_CONTROL_TYPE, gcpBonoboControlClass))

#define GCP_BONOBO_IS_CONTROL(o)          (G_TYPE_CHECK_INSTANCE_TYPE ((o), GCP_BONOBO_CONTROL_TYPE))
#define GCP_BONOBO_IS_CONTROL_CLASS(k)    (G_TYPE_CHECK_CLASS_TYPE ((k), GCP_BONOBO_CONTROL_TYPE))
#define GCP_BONOBO_CONTROL_GET_CLASS(o)   (G_TYPE_INSTANCE_GET_CLASS ((o), GCP_BONOBO_CONTROL_TYPE, gcpBonoboControlClass))

typedef struct _gcpBonoboControl         gcpBonoboControl;
typedef struct _gcpBonoboControlClass    gcpBonoboControlClass;

struct _gcpBonoboControl {
	BonoboControl control;
	PersistStream *ps;

	gcpDocument *pDoc;
};

struct _gcpBonoboControlClass {
	BonoboControlClass parent_class;
};

static GType gcp_bonobo_control_get_type(void);
static gcpBonoboControl *gcp_bonobo_control_new(gcpDocument *pDoc);
static gcpBonoboControl *gcp_bonobo_control_construct(gcpBonoboControl *control, gcpDocument *pDoc);

static GObjectClass *gcp_bonobo_control_parent_class;

static void
gcp_bonobo_control_destroy (BonoboObject *object)
{
	g_return_if_fail (object != NULL);
	g_return_if_fail (GCP_BONOBO_IS_CONTROL (object));
	BONOBO_OBJECT_CLASS (gcp_bonobo_control_parent_class)->destroy (object);
}

static void
gcp_bonobo_control_finalize (GObject *object)
{
	gcpBonoboControl *control;
	g_return_if_fail (object != NULL);
	g_return_if_fail (GCP_BONOBO_IS_CONTROL (object));

	control = GCP_BONOBO_CONTROL (object);
	delete control->pDoc;
	
	G_OBJECT_CLASS (gcp_bonobo_control_parent_class)->finalize (object);
}

static void
gcp_bonobo_control_activate (BonoboControl *object, gboolean state)
{
	gcpBonoboControl *control;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GCP_BONOBO_IS_CONTROL (object));

	control = GCP_BONOBO_CONTROL (object);

/*	if (state) {
		Bonobo_UIContainer ui_container;

		ui_container = bonobo_control_get_remote_ui_container (BONOBO_CONTROL (control), NULL);
		if (ui_container != CORBA_OBJECT_NIL) {
			gcp_bonobo_control_set_ui_container (control, ui_container);
			bonobo_object_release_unref (ui_container, NULL);
		}
	} else
		gcp_bonobo_control_unset_ui_container (control);*/

	if (BONOBO_CONTROL_CLASS (gcp_bonobo_control_parent_class)->activate)
		BONOBO_CONTROL_CLASS (gcp_bonobo_control_parent_class)->activate (object, state);
}

void gcp_bonobo_control_class_init (gcpBonoboControlClass *klass)
{
	GObjectClass *gobject_class = (GObjectClass *)klass;
	BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)klass;
	BonoboControlClass *control_class = (BonoboControlClass *)klass;

	gcp_bonobo_control_parent_class = (GObjectClass *)g_type_class_peek_parent ((GObjectClass *)klass);

	bonobo_object_class->destroy = gcp_bonobo_control_destroy;
	gobject_class->finalize = gcp_bonobo_control_finalize;
	control_class->activate = gcp_bonobo_control_activate;
}

static void
gcp_bonobo_control_init (gcpBonoboControl *control)
{
}

BONOBO_TYPE_FUNC (gcpBonoboControl, BONOBO_TYPE_CONTROL, gcp_bonobo_control);

gcpBonoboControl* gcp_bonobo_control_construct(gcpBonoboControl *control, gcpDocument *pDoc)
{
	GtkWidget             *widget;
	BonoboPropertyBag     *pb;
	BonoboPropertyControl *pc;
	
	g_return_val_if_fail (pDoc != NULL, NULL);
	g_return_val_if_fail (control != NULL, NULL);
	g_return_val_if_fail (GCP_BONOBO_IS_CONTROL (control), NULL);

	widget = pDoc->GetView()->CreateNewWidget();

	bonobo_control_construct (BONOBO_CONTROL (control), widget);
	
	control->ps = persist_stream_new();
	control->ps->pDoc = pDoc;
	pDoc->SetBonoboPersist((BonoboPersist*)control->ps);
	bonobo_object_add_interface((BonoboObject*)control, (BonoboObject*)control->ps);

	return control;
}

gcpBonoboControl *
gcp_bonobo_control_new (gcpDocument *pDoc)
{
	gcpBonoboControl *control;

	control = (gcpBonoboControl*) g_object_new (GCP_BONOBO_CONTROL_TYPE, NULL);

	return gcp_bonobo_control_construct (control, pDoc);
}


BonoboObject * gchempaint_factory(BonoboGenericFactory * fact, const char *component_id, void* data)
{
	g_return_val_if_fail (fact != NULL, NULL);
	g_return_val_if_fail (component_id != NULL, NULL);

	BonoboObject *retval;
	gcpDocument* pDoc =  new gcpDocument();
	if (!pDoc) return NULL;
	if (!strcmp (component_id, "OAFIID:gchempaint_control"))
	{
		retval = (BonoboObject*) gcp_bonobo_control_new(pDoc);
		if (!retval)  delete pDoc;
	}
	else if (!strcmp (component_id, "OAFIID:gchempaint_item"))
	{
		g_warning ("GChemPaint Item is not implemented yet");
		return NULL;
	}
	else
	{
		g_warning ("Unknown IID `%s' requested", component_id);
		delete pDoc;
		return NULL;
	}

	return retval;
}

